## functional.py provides support for a functional style of Python programming.
## Copyright (C) 2000 Bryn Keller

## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public
## License as published by the Free Software Foundation; either
## version 2.1 of the License, or (at your option) any later version.

## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## Lesser General Public License for more details.

## You should have received a copy of the GNU Lesser General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


"""
functional.py provides support for a functional style of Python programming.

It includes support for closures, curried functions, lazy expressions, lazy tuples
(functional programming languages call these lazy lists, but since lists are
mutable in Python, tuples are closer in meaning), and lazy equivalents for map,
filter, reduce, and zip. Also some higher-order functions for composing functions and such.
"""

from types import StringType
import sys
from copy import copy
try:
    #No 'new' module in JPython, so closures and lazyfilter (which
    #uses closures) will not be available.
    import new
except:
    class NewErrorMessage:
        def __getattr__(self, name):
            raise TypeError, "The 'new' module could not be imported, so this feature cannot be used."
    new = NewErrorMessage()
import traceback



def getStackFrame():
    """
    Utility function to return the current stack frame.
    """
    try:
        raise RuntimeError
    except:
        tb = sys.exc_info()[2]
        frame = tb.tb_frame.f_back
    return frame

def isLazy(thing):
    """
    Returns 1 if *thing* is a Lazy expression, 0 otherwise.
    """
    try:
        return isinstance(thing, Lazy)
    except:
        return 0

class Lazy:
    """
    Lazy expressions are expressions which are not computed until needed.
    Many functional languages (Haskell, Scheme, OCaml, etc.) provide support
    for this as a language feature, but a large class of those things can
    be simulated in Python with this class and/or LazyTuple.
    Things that will force a lazy expression *expr* to be evaluated:
        - *expr*.eval()
        - any value comparison, such as *expr* < 5, or *expr* == "abc"
        - any sort of operation such as slicing, getattrs, or mathematical
          opertions, **unless the other operands involved are also lazy.**
        - calling str(*expr*). Same for int, float, long.
    A lazy expression is evaluated in the namespaces of the code which constructed
    it, unless the defaults are overridden. For example:

    a = 5
    laz = Lazy("a * 5")
    a = 7
    assert laz == 25

    #Now laz can be used in any context, and will always evaluate to 25, even if
    #a changes or goes out of scope.
    """
    def __init__(self, code, globs = {}, locs = {}):
        """
        *code* can be either a code object or a string. Either way, it should
        be an expression, not a statement. If the *globs* or *locs* params are
        omitted, the global and local namespaces of the caller will be used
        for evaluation.
        """
        if type(code) == StringType:
            code = compile(code, 'Lazy expr: ' + code, 'eval')
        self.__dict__['_code'] = code
        if not globs or not locs:
            frame = getStackFrame().f_back
            l, g = frame.f_locals, frame.f_globals
            if not locs:
                locs = l.copy()
            if not globs:
                globs = g.copy()
        self.__dict__['_globs'] = globs
        self.__dict__['_locs'] = locs

    def eval(self):
        """
        Forces (and returns) the evaluation of the expression.
        """
        if not self.__dict__.has_key('_value'):
            self.__dict__['_value'] = eval(self._code, self._globs, self._locs)
            #Delete namespaces so we don't prevent GC on their contents...
            del self.__dict__['_globs']
            del self.__dict__['_locs']
        return self.__dict__['_value']

    def __str__(self):
        """
        Will cause evaluation.
        """
        return str(self.eval())

    def __repr__(self):
        """
        Does **not** cause evaluation.
        """
        return self._code.co_filename

    def __len__(self):
        """
        Will cause evaluation.
        """
        return len(self.eval())

    def __mul__(self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() * other.eval()", locs = locals())
        return self.eval() * other

    def __rmul__(self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() * self.eval()")
        return other * self.eval()

    def __div__(self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() * other.eval()")
        return self.eval()/other

    def __rdiv__(self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval()/self.eval()")
        return other/self.eval()

    def __mod__(self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() % self.eval()")
        return other % self.eval()

    def __rmod__(self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() % other.eval()")
        return self.eval() % other

    def __divmod__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("divmod(other.eval(),self.eval())")
        return divmod(other, self.eval())

    def __pow__ (self, other, modulo):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("pow(self.eval(), other.eval(), modulo)")
        return pow(self.eval(), other, modulo)

    def __lshift__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() << other.eval()")
        return self.eval() << other

    def __rshift__ (self, other) :
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() >> other.eval()")
        return self.eval() >> other

    def __and__ (self, other) :
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() & other.eval()")
        return self.eval() & other

    def __xor__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() ^ other.eval()")
        return self.eval() ^ other
    def __or__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("self.eval() | other.eval()")

    def __radd__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() + self.eval()")
        return other + self.eval()

    def __rsub__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() - self.eval()")
        return other - self.eval()

    def __rdivmod__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("divmod(other.eval(), self.eval())")
        return divmod(other + self.eval())

    def __rpow__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("pow(other.eval(), self.eval())")
        return pow(other, self.eval())

    def __rlshift__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() << self.eval()")
        return other << self.eval()

    def __rrshift__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() >> self.eval()")
        return other >> self.eval()

    def __rand__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() & self.eval()")
        return other & self.eval()

    def __rxor__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() ^ self.eval()")
        return other ^ self.eval()

    def __ror__ (self, other):
        """
        Will cause evaluation, unless *other* is also lazy.
        """
        if isLazy(other):
            return Lazy("other.eval() | self.eval()")
        return other | self.eval()

    def __neg__ (self):
        """
        Will cause evaluation.
        """
        return -self.eval()

    def __pos__ (self):
        """
        Will cause evaluation.
        """
        return +self.eval()

    def __abs__ (self):
        """
        Will cause evaluation.
        """
        return abs(self.eval())

    def __invert__ (self):
        """
        Will cause evaluation.
        """
        return not self.eval()

    def __complex__ (self):
        """
        Will cause evaluation.
        """
        return complex(self.eval())

    def __oct__ (self):
        """
        Will cause evaluation.
        """
        return oct(self.eval())

    def __hex__ (self):
        """
        Will cause evaluation.
        """
        return hex(self.eval())

    def __getitem__(self, ntx):
        """
        Does **not** cause evaluation.
        """
        if isLazy(ntx):
            return Lazy("self.eval()[ntx.eval()]")
        else:
            return Lazy("self.eval()[ntx]")

    def __getslice__(self, i, j):
        """
        Does **not** cause evaluation.
        """
        return Lazy("self.eval()[i:j]")

    def __delslice__(self, i, j):
        """
        Will cause evaluation.
        """
        del self.eval()[i:j]

    def __setitem__(self, key, value):
        """
        Will cause evaluation.
        """
        self.eval()[key] = value

    def __cmp__(self, other):
        """
        Will cause evaluation.
        """
        return cmp(self.eval(), other)

    def __getattr__(self, name):
        """
        Does **not** cause evaluation.
        """
        if name == '__coerce__':
            raise AttributeError, name
        return Lazy("getattr(self.eval(), name)")

    def __setattr__(self, name, value):
        """
        Will cause evaluation.
        """
        setattr(self.eval(), name, value)

    def __delattr__(self, name):
        """
        Will cause evaluation.
        """
        delattr(self.eval(), name)

    def __int__(self):
        """
        Will cause evaluation.
        """
        return int(self.eval())

    def __float__(self):
        """
        Will cause evaluation.
        """
        return float(self.eval())

    def __long__(self):
        """
        Will cause evaluation.
        """
        return long(self.eval())

    def __copy__(self):
        """
        Does **not** cause evaluation.
        """
        import copy
        return Lazy("copy.copy(self.eval())")

    def __deepcopy__(self, dict):
        """
        Does **not** cause evaluation.
        """
        import copy
        return Lazy("copy.deepcopy(self.eval())")

class Uncomputed:
    """
    Placeholder value used in LazyTuples when an index is known to exist, but
    does not need to be computed yet.
    """
    pass


class LazyTuple:
    """
    Lazy tuples (equivalent to lazy lists in functional languages) are sequences
    whose values are calculated on demand. In a normal (strict) tuple, every
    value in the tuple must be known at the time it is constructed, and it has a
    definite length. Not so with LazyTuples, which may have potentially infinite
    length, and the value at a given index is computed only when needed for some
    other calculation.

    In this implementation, a function is defined which takes two arguments, an
    index, and the LazyTuple the index is being done on. Whatever this function
    returns is the value of the LazyTuple at that index. The function will only
    be called once for each index, and the cached value will be used in
    subsequent indexing operations.

    For example:

    #Build a lazy list of squares:
    def square(index, seq):
        return index * index

    >>>laz = LazyList(itemFunc = square)
    >>>print laz[2]
    4
    >>>

    #But this is a simple case which can be extrapolated directly from the
    #index. A better example:

    def factorial(index, seq):
        if index == 0:
            return 1
        else:
            return seq[index -1] * index

    >>>laz = LazyList(itemFunc = factorial)
    >>>laz[3]
    6
    >>>

    In this last example, laz[3] caused factorial to be called with index == 3,
    which in turn forced the evaluation (because it's necessary for factorial
    construction) of all the indices prior to 3. In the first example, no
    other indices need to be computed.

    Note that neither of these examples *terminates*, that is they are both
    of infinite length (actually they'll overflow from integer multiplication
    eventually, but's that's not important for this discussion). When dealing
    with infinite tuples, a few rules apply:

    - *lazytuple*.isTerminating() will return 1 if the tuple has definite length,
        0 otherwise.
    - len(*lazytuple*) will raise a RuntimeError if the list is infinite.
    - if *lazytuple*: will always be true for an infinite tuple.
    - LazyTuples may be compared normally.
    - iteration over a LazyTuple with for works fine, but if the tuple doesn't
      terminate, neither will your loop!

    """
    def __init__(self, itemFunc = None, length = -1, sliceFrom = None, sliceStart = 0):
        """
        *itemFunc* is a function which takes two arguments, *index* and
        *sequence*, and is used to generate the value at an arbitrary
        index. If *length* is supplied, then this is a bounded (finite)
        LazyTuple. *sliceFrom* and *sliceStart* are used internally when
        slices are taken from a LazyTuple. Storage space is never duplicated
        for LazyTuples and slices taken from them.
        """
        self._itemFunc = itemFunc
        self._memo = []
        self._evaluated = 0
        self._sliceStart = sliceStart
        self._sliceFrom = sliceFrom
        self._length = length
        self._evaluated = 0

    def isTerminating(self):
        """
        Return 0 if this is a finite tuple, 1 if it is infinitely long.
        """
        return self._length > 0

    def __getitem__(self, i):
        if self._length > 0 and i >= self._length:
            raise IndexError, i
        elif self._length < 1 and i < 0:
            raise IndexError, i
        if not self._sliceFrom is None:
            return self._sliceFrom[self._sliceStart + i]
        if len(self._memo) > i:
            val = self._memo[i]
            if not val is Uncomputed:
                return self._memo[i]
        val = self._itemFunc(i, self)
        if len(self._memo) <= i:
            self._memo.extend([Uncomputed] * ((i+1) - len(self._memo)))
        self._memo[i] = val
        return val

    def __str__(self):
        val = ""
        try:
            for i in range(3):
                if val:
                    val = val + ", "
                val = val + str(self[i])
        except IndexError:
            pass
        else:
            val = val + ", .."

        return "LazyTuple (" + val + ")"

    def __getslice__(self, i, j):
        if j != sys.maxint:
            length = j - i
        elif self._length > 0:
            length = self._length - i
        else:
            length = -1
        if self._length > 0:
            if i >= self._length:
                raise IndexError, i
            if j >= self._length and j != sys.maxint:
                raise IndexError, j
        if self._sliceFrom:
            return LazyTuple(sliceFrom = self._sliceFrom, length = length,
                             sliceStart = self._sliceStart + i)
        return LazyTuple(sliceFrom = self, length = length, sliceStart = i)

    def __len__(self):
        if not self.isTerminating():
            raise RuntimeError, "Non-terminating structure, cannot evaluate len()."
        if self._length > 0:
            return self._length
        else:
            self.eval()
            return len(self._memo)

    def __nonzero__(self):
        return 1

    def __cmp__(self, other):
        if self is other:
            return 0
        if isinstance(other, LazyTuple):
            if self.isTerminating():
                if not other.isTerminating():
                    return -1
            elif other.isTerminating():
                return 1
        elif type(other) == type(()):
            if not self.isTerminating():
                return 1
            else:
                if cmp(self._length, len(other)):
                    return cmp(self._length, len(other))
                else:
                    for sthing, othing in lazyzip(self, other):
                        if cmp(sthing, othing):
                            return cmp(string, othing)
                    return 0
        else:
            return 1

    def __mul__(self, other):
        if not self.isTerminating():
            return self
        return self.eval() * other

    def __add__(self, other):
        if not self.isTerminating():
            return self
        return self.eval() + other


    def eval(self):
        if not self._length:
            raise RuntimeError, "Cannot eval a non-terminating sequence."
        if not self._evaluated:
            i = 0
            while 1:
                try:
                    self[i]
                    i = i + 1
                except IndexError:
                    self._evaluted = 1
                    break
        if self._sliceFrom:
            retval = self._sliceFrom._memo[self._sliceStart:self._sliceStart + self._length]
        else:
            retval = self._memo
        return tuple(retval)





def integers(index, seq, startFrom = 0, step = 1):
    """
    An index function for LazyTuples of consecutive integers.
    """
    if index == 0:
        return startFrom
    else:
        return seq[index -1] + step

def naturals(index, seq, startFrom = 1, step = 1):
    """
    An index function for LazyTuples of consecutive natural numbers.
    """
    assert startFrom > 0
    return integers(index, seq, startFrom, step)

def lazymap(func, seq):
    """
    Lazy equivalent for the map builtin function.
    lazymap returns a LazyTuple whose contents are computed on demand
    by applying *func* to seq[i], where i is the index that's being accessed.
    """
    term = 1
    length = -1
    newItemFunc = lambda index, sequence, orig = seq, f = func:f(orig[index])
    if not (isinstance(seq, LazyTuple) and not seq.isTerminating()):
        length = len(seq)
    return LazyTuple(itemFunc = newItemFunc, length = length)

def lazyfilter(func, seq):
    """
    Lazy equivalent for the filter builtin function.
    lazyfilter returns a LazyTuple whose contents are computed on demand
    by filtering as much of the original sequence as necessary to reach
    a value for the necessary index.
    """
    seqIndex = 0
    def filterhelper(index, sequence):
        global seqIndex
        while 1:
            val = seq[seqIndex]
            seqIndex = seqIndex + 1
            if func(val):
                return val
    return LazyTuple(itemFunc = closure(filterhelper), length = -1)

def lazyreduce(func, seq):
    """
    Lazy equivalent for the reduce builtin function.
    lazyreduce can only be applied to terminating (non-infinite) tuples.
    """
    if isinstance(seq, LazyTuple) and not seq.isTerminating():
        raise RuntimeError, "Cannot reduce infinite tuple."
    return Lazy("reduce(func, tuple(seq))")

def lazyzip(*seqs):
    """
    Lazy equivalent for the zip builtin function.
    """
    def newItemFunc(index, seq, origseqs = seqs):
        tup = []
        for orig in origseqs:
            tup.append(orig[index])
        return tuple(tup)
    return LazyTuple(itemFunc = newItemFunc)

def even(x):
    """
    Returns 1 if x is even, 0 otherwise.
    """
    return x % 2 == 0

def positive(x):
    """
    Returns 1 if x is positive, 0 otherwise.
    """
    return x > 0

def negative(x):
    """
    Returns 1 if x is negative, 0 otherwise.
    """
    return x < 0

class Ref:
    """
    A simple class to hold a value. Useful when one wishes to share a value
    between functions, or otherwise access the value from a local scope without
    using a 'global' statement to avoid rebinding the name locally.
    """
    def __init__(self, val):
        self.val = val


def closure(func, locs = {}):
    """
    Returns a function whose global namespace is a **copy of** *func*'s
    globals, augmented with the local variables of the calling function.
    This is a common requirement for functional programming. This can
    be duplicated in standard Python using default arguments, but it
    is generally more cumbersome. Example:

    #A function which returns a function which returns 1 if an item is
    #in *sequence*, 0 otherwise
    def contained(sequence):
        def contained_helper(x):
            return x in sequence
        return closure(contained_helper)

    >>>lst = [1,2,3]
    >>>is123 = contained(lst)
    >>>is123(2)
    1
    >>>


    #An example of 'static' variables, somewhat like in C:
    def bindCallCount(func):
        callCount = ref(0)
        return closure(func)

    #Note this function uses a variable, callCount, that is not defined.
    def printAndIncrement():
        print callCount.val
        callCount.val = callCount.val + 1

    #But with closures, we can make this function work:
    >>>printAndInc = bindCallCount(printAndIncrement)
    >>>printAndInc()
    0
    >>>printAndInc()
    1
    >>>

    So callCount is preserved across function invocations, but is not taking
    up space in the module namespace.
    """
    frame = getStackFrame().f_back
    if not locs:
        locs = frame.f_locals
    globs = {}
    globs.update(func.func_globals)
    globs.update(locs)
    newfunc = new.function(func.func_code, globs)
    globs[func.__name__] = newfunc
    return newfunc



class Blank:
    """
    When passed in a call to curry, or to a curried function, this is treated as a placeholder
    for future calls to replace.
    """
    pass

def blend(func, original, mix):
    """
    This function returns a list based on *original*, with any elements that
    *func* returns nonzero for replaced one by one with items from *mix*.
    Any unused items in *mix* appear at the end of the new list. If there are
    items which should be replaced, but there are no more items in *mix*, then
    those items are left unchanged.
    """
    mixIndex = 0
    build = []
    mixlen = len(mix)
    for item in original:
        if mixIndex < mixlen and func(item):
            build.append(mix[mixIndex])
            mixIndex = mixIndex + 1
        else:
            build.append(item)
    if mixIndex < mixlen:
        build.extend(list(mix[mixIndex:]))
    return build

def getMinimumArgCount(func):
    try:
        if hasattr(func, 'im_func'):
            count = func.im_func.func_code.co_argcount -1
            count = count - len(func.im_func.func_defaults)
        else:
            count = func.func_code.co_argcount
            count = count - len(func.func_defaults)
    except:
        pass
    return count

class curry:
    """
    A curried function (named after the person who pioneered the idea,
    Haskell Currry) is one which some arguments are already defined,
    something like default arguments in Python, but much more
    general. When you curry a function with some arguments, the
    result is a function which requires only those arguments that
    haven't been supplied already. For example:

    def addTwo(x, y):
        return x + y

    addFive = curry(addTwo, 5)

    Now addFive is a function which takes one argument, and returns
    that argument added to 5, since x is pre-bound to 5 by the curry().

    In functional languages, currying must take place either from the
    beginning of the argument list, or the end (this is sometimes called
    an rcurry, or reverse curry). In Python, with keyword arguments,
    and *args and **kwargs, the situation is a little more complicated,
    so these are the rules:

    - All currying starts from the left side of the argument list.
    - Keyword arguments can be curried.
    - If you wish to supply an argument that occurs later in the
      paramter list without supplying the one before it, you can
      supply the special value Blank in the places you don't want
      filled. For example:
          addFive = curry(addTwo, Blank, 5)
      would result in the 5 argument being bound to y instead of x.

    If curry() can determine the minimum number of arguments for the
    function (func.func_code.co_argcount is present), then calls to
    the curried function with fewer than the remaining required arguments
    will only return a new curried function with the new arguments
    bound as well. For example:

    def threeArgs(x, y, z):
        return x + y + z

    twoArgs = curry(threeArgs, 5)

    Now twoArgs is a function which takes two arguments, and adds them to 5.
    If we now do:

    NewThing = twoArgs(20)

    NewThing is now a function which takes one argument, and returns that value
    plus 20 + 5. As soon as the function has accumulated enough arguments to
    reach the required argument count of the original function, the original
    function is called with all the accumulated arguments.

    In the case of explicity calling curry, however, if all arguments are
    supplied, the function is not called, rather a no-arguments function is
    returned. For example:

    oneArg = curry(twoArgs, 5)
    oneArg(5)

    would result in a call to twoArgs with arguments 5, 5. However:

    noArgs = curry(twoArgs, 5, 5)

    does not call twoArgs, until noArgs is called:

    noArgs()

    which again calls twoArgs, passing 5, 5 as the arguments.

    When applied to a function which has default arguments, only a number of
    arguments equal to the number of basic (non-defaulted) paramters will be
    required before the function is executed. For example:

    >>> def defaultTest(x, y, z, q = 4):
    ... 	return x, y, z, q
    ...
    >>> c = curry(defaultTest, 1, 2, 3)
    >>> c()
    (1, 2, 3, 4)
    >>>

    Of course, this doesn't prevent default arguments from being supplied either
    initially or at a later time, so long as that minimum count hasn't been met.
    Example:
    >>> c = curry(defaultTest, 1)
    >>> c(q = 7)
    <functional.curry instance at 1317a70>
    >>> c(2,3)
    (1, 2, 3, 4)
    >>>
    """
    def __init__(self, func, *args, **kwargs):
        self._args = args
        self._kwargs = kwargs
        self._func = func
        self._minCount = getMinimumArgCount(func)

    def __call__(self, *args, **kwargs):
        buildArgs = blend(lambda x:x is Blank, self._args, args)
        buildKwargs = self._kwargs.copy()
        buildKwargs.update(kwargs)
        hasblank = Blank in buildArgs or Blank in buildKwargs.values()
        argcount = len(buildArgs) + len(buildKwargs)
        if hasblank or (argcount < self._minCount):
            cur = curry(self._func)
            cur._args = buildArgs
            cur._kwargs = buildKwargs
            return cur
        else:
            return apply(self._func, buildArgs, buildKwargs)

class rcurry(curry):
    """
    *rcurry* works much like *curry*, but applies its arguments to the
    other end of the argument list. For example:
    >>> def threeArgs(x, y, z):
    ... 	return x, y, z
    ...
    >>> rc = rcurry(threeArgs, 2, 3)
    >>> rc(1)
    (1, 2, 3)
    >>>
    """
    def __init__(self, func, *args, **kwargs):
        apply(curry.__init__, (self, func) + args, kwargs)
        argdiff = self._minCount - len(self._args)
        if len(self._args) < self._minCount:
            self._args = ((Blank,) * argdiff) + self._args



class compose:
    """
    Takes any number of functions, and when called, applies them in
    reverse order. The first function to be called (the last to be passed
    in the call to compose()) is called with the arguments, and the
    other functions are called with the results of the previous function
    as its arguments. For example:

    def f(x):
        return x * 2

    def g(x):
        return x + 5

    h = compose(f, g)

    h(3) will now equal f(g(3)), or 16.
    """
    def __init__(self, *args):
        args = list(args)
        args.reverse()
        self._head = args[0]
        self._tail = args[1:]
    def __call__(self, *args, **kwargs):
        ret = apply(self._head, args, kwargs)
        for func in self._tail:
            ret = func(ret)
        return ret

class joinfuncs:
    """
    Takes a number of functions, and returns a function which, when called,
    returns a tuple of all the values the original functions would return
    if called with those arguments. Example:

    def timesFive(x):
        return x * 5

    def square(x):
        return x * X

    >>>myfunc = joinfuncs(timesFive, square, str)
    >>>myfunc(2)
    (10, 4, '2')

    """
    def __init__(self, *args):
        self._funcs = args

    def __call__(self, *args, **kwargs):
        lst = []
        for func in self._funcs:
            lst.append(apply(func, args, kwargs))
        return tuple(lst)

class complement:
    """
    When called, returns the logical not of a call to *func* with the same
    arguments.
    """
    def __init__(self, func):
        self._func = func

    def __call__(self, *args, **kwargs):
        return not apply(self._func, args, kwargs)

class disjoin:
    """
    Takes the functions *funcs*, and when called will return 1 if any of
    *funcs* returns nonzero when applied to the arguments the disjoin was called with.
    This is basically equivalent to logical or-ing calls to all the functions in *funcs*.
    Note that if one function returns nonzero, the remaining functions are not
    evaluated. Example:

    #We need to find whether something is a sequence, other than a string.
    isTuple = lambda x:type(x) == type(())
    isList = lambda x:type(x) == type([])
    isUserSeq = lambda x:hasattr(x, '__getslice__')

    isSequence = disjoin(isTuple, isList, isUserSeq)
    """
    def __init__(self, *funcs):
        self._funcs = funcs

    def __call__(self, *args, **kwargs):
        for func in self._funcs:
            if apply(func, args, kwargs):
                return 1
        return 0

class conjoin:
    """
    Takes the functions *funcs*, and when called will return 1 if all of
    *funcs* return nonzero when applied to the arguments the conjoin was called with.
    This is basically equivalent to logical and-ing all the functions in *funcs*.
    Example:

    #Function to decide if a number is greater than 5 and less than 10...
    gt5 = lambda x:x > 5
    lt10 = lambda x:x < 10

    between_5_and_10 = conjoin(gt5, lt10)
    """
    def __init__(self, *funcs):
        self._funcs = funcs
    def __call__(self, *args, **kwargs):
        for func in self._funcs:
            if not apply(func, args, kwargs):
                return 0
        return 1



def when(expr, truepart, falsepart = None, force = 0):
    """
    Functional equivalent for the if(else) statement.
    Note that unlike a real if, both truepart and falsepart will be evaluated,
    unless of couse they are Lazy. If they are lazy, only the selected
    expression will be returned. To get the selected lazy expression to be
    evaluated, call with *force* = 1 Example:

    >>> def printAndReturn(arg):
    ... 	print arg
    ... 	return arg
    ...
    >>> when(0, printAndReturn(0), printAndReturn(1))
    0
    1
    1
    #Note printAndReturn was called twice.

    >>> when(0, Lazy("printAndReturn(0)"), Lazy("printAndReturn(1)"))
    Lazy expr: printAndReturn(1)
    >>> when(0, Lazy("printAndReturn(0)"), Lazy("printAndReturn(1)"), 1)
    1
    1
    #because of the last (force) parameter, the chosen lazy expression was
    #evaluated before returning.
    >>>
    """
    if expr:
        retval = truepart
    else:
        retval = falsepart
    if isLazy(retval) and force:
        retval = retval.eval()
    return retval


